@@ -182,7 +182,7 @@ class Simulation1d(myokit.CModule):
         # Unique simulation id
         Simulation1d._index += 1
         module_name = 'myokit_sim1d_' + str(Simulation1d._index)
-        module_name += '_' + str(myokit._pid_hash())
+        module_name += '_' + str(myokit.pid_hash())
 
         # Arguments
         args = {
@@ -196,8 +196,8 @@ class Simulation1d(myokit.CModule):
 
         # Debug
         if myokit.DEBUG:
-            print(self._code(fname, args,
-                             line_numbers=myokit.DEBUG_LINE_NUMBERS))
+            print(self._code(
+                fname, args, line_numbers=myokit.DEBUG_LINE_NUMBERS))
             import sys
             sys.exit(1)
 
@@ -336,17 +336,20 @@ class Simulation1d(myokit.CModule):
         tmax = tmin + duration
 
         # Gather global variables in model
-        g = [self._model.time().qname()]
+        global_vars = [self._model.time().qname()]
+        pace = self._model.binding('pace')
+        if pace is not None:
+            global_vars.append(pace.qname())
 
         # Parse log argument
+        allowed = myokit.LOG_STATE + myokit.LOG_BOUND + myokit.LOG_INTER
         log = myokit.prepare_log(
             log,
             self._model,
             dims=(self._ncells,),
-            global_vars=g,
+            global_vars=global_vars,
             if_empty=myokit.LOG_STATE + myokit.LOG_BOUND,
-            allowed_classes=myokit.LOG_STATE + myokit.LOG_BOUND
-            + myokit.LOG_INTER,
+            allowed_classes=allowed,
         )
 
         # Get event tuples
