@@ -71,6 +71,7 @@ bound_variables = model.prepare_bindings({
 # Get equations
 equations = model.solvable_order()
 ?>
+#define PY_SSIZE_T_CLEAN
 #include <Python.h>
 #include <stdio.h>
 #include <math.h>
@@ -211,7 +212,7 @@ static long engine_steps = 0;
 <?
 for var in model.variables(state=False, deep=True):
     if var.is_literal():
-        print('static realtype ' + v(var) + ' = ' + myokit.strfloat(var.rhs().eval()) + ';')
+        print('static realtype ' + v(var) + ' = ' + myokit.float.str(var.rhs().eval()) + ';')
     else:
         print('static realtype ' + v(var) + ';')
 ?>
@@ -373,6 +374,24 @@ log_add(PyObject* log_dict, PyObject** logs, realtype** vars, int i, const char*
     return added;
 }
 
+/*
+ * Error and warning message handler for CVODE.
+ * Error messages are already set via check_cvode_flag, so this method
+ * suppresses error messages.
+ * Warnings are passed to Python's warning system, where they can be
+ * caught or suppressed using the warnings module.
+ */
+void
+ErrorHandler(int error_code, const char *module, const char *function,
+             char *msg, void *eh_data)
+{
+    char errstr[1024];
+    if (error_code > 0) {
+        sprintf(errstr, "CVODE: %s", msg);
+        PyErr_WarnEx(PyExc_RuntimeWarning, errstr, 1);
+    }
+}
+
 /*
  * Simulation variables
  */
@@ -794,6 +813,10 @@ for var in model.variables(deep=True, state=False, bound=False, const=False):
     #endif
     if (check_cvode_flag((void*)cvode_mem, "CVodeCreate", 0)) return sim_clean();
 
+    /* Set error and warning-message handler */
+    flag_cvode = CVodeSetErrHandlerFn(cvode_mem, ErrorHandler, NULL);
+    if (check_cvode_flag(&flag_cvode, "CVodeInit", 1)) return sim_clean();
+
     /* Initialise solver memory, specify the rhs */
     flag_cvode = CVodeInit(cvode_mem, rhs, engine_time, y);
     if (check_cvode_flag(&flag_cvode, "CVodeInit", 1)) return sim_clean();
