@@ -79,10 +79,11 @@ mcl_flag2(const char* msg, const cl_int flag)
     if(strcmp(msg, "")) {
         sprintf(sub, " (%s)", msg);
     } else {
-        sprintf(sub, "");
+        sub[0] = 0;
     }
 
     switch(flag) {
+        // OpenCL 1.0 Errors
         case CL_DEVICE_NOT_FOUND:
             sprintf(err, "OpenCL error%s: CL_DEVICE_NOT_FOUND", sub);
             break;
@@ -230,6 +231,13 @@ mcl_flag2(const char* msg, const cl_int flag)
         case CL_INVALID_PROPERTY:
             sprintf(err, "OpenCL error%s: CL_INVALID_PROPERTY", sub);
             break;
+        // OpenCL 1.1 etc. codes can not be assumed to be defined
+        // Might be good to have ifdefs or something
+        // OpenCL extensions
+        case -1001:     // CL_PLATFORM_NOT_FOUND_KHR
+            sprintf(err, "OpenCL error%s: CL_PLATFORM_NOT_FOUND_KHR", sub);
+            break;
+        // Unknown error
         default:
             sprintf(err, "Unknown OpenCL error%s: %i", sub, (int)flag);
             break;
@@ -254,7 +262,7 @@ mcl_flag(const cl_int flag)
  * Arguments:
  *  PyObject* platform  A string representing the platform, or None
  *  PyObject* device    A string representing the device, or None
- *  cl_platform_id* pid The returned cl_platform_id, or NULL
+ *  cl_platform_id* pid The returned cl_platform_id
  *  cl_device_id* did   The returned cl_device_id
  * The returned value is 0 if no error occurred, 1 if an error did occur. In
  *  this case a python error message will also be set.
@@ -282,6 +290,10 @@ int mcl_select_device(
     // OpenCL ints for iterating
     cl_uint i, j;
 
+    // By default, don't recommend a platform or device
+    *pid = NULL;
+    *did = NULL;
+
     // Check input
     const char* pname;
     const char* dname;
@@ -330,9 +342,6 @@ int mcl_select_device(
     // Platform unspecified
     if (platform == Py_None) {
 
-        // Don't recommend a platform
-        *pid = NULL;
-
         // No platform or device specified
         if (device == Py_None) {
 
@@ -340,7 +349,8 @@ int mcl_select_device(
             for (i=0; i<n_platforms; i++) {
                 flag = clGetDeviceIDs(platform_ids[i], CL_DEVICE_TYPE_GPU, 1, device_ids, &n_devices);
                 if(flag == CL_SUCCESS) {
-                    // Set selected device and return.
+                    // Set selected platform and device and return.
+                    *pid = platform_ids[i];
                     *did = device_ids[0];
                     return 0;
                 } else if(flag != CL_DEVICE_NOT_FOUND) {
@@ -352,7 +362,8 @@ int mcl_select_device(
             for (i=0; i<n_platforms; i++) {
                 flag = clGetDeviceIDs(platform_ids[i], CL_DEVICE_TYPE_ALL, 1, device_ids, &n_devices);
                 if(flag == CL_SUCCESS) {
-                    // Set selected device and return.
+                    // Set selected platform and device and return.
+                    *pid = platform_ids[i];
                     *did = device_ids[0];
                     return 0;
                 } else if(flag != CL_DEVICE_NOT_FOUND) {
@@ -376,7 +387,8 @@ int mcl_select_device(
                         if(mcl_flag(flag)) return 1;
                         trim(name);
                         if (strcmp(name, dname) == 0) {
-                            // Set selected device and return.
+                            // Set selected platform device and return.
+                            *pid = platform_ids[i];
                             *did = device_ids[j];
                             return 0;
                         }
@@ -483,34 +495,271 @@ mcl_round_total_size(const int ws_size, const int total_size)
     return size;
 }
 
-/* Memory used by mcl_device_info */
-static PyObject* platforms = NULL;  // Tuple of platform dicts
-static PyObject* platform = NULL;   // Temporary platform dict
-static PyObject* devices = NULL;    // Temporary tuple of devices
-static PyObject* device = NULL;     // Temporary device dict
-static PyObject* items = NULL;      // Temporary tuple of item sizes
-static PyObject* val;               // Temporary dictionary value
-static size_t* work_item_sizes;     // Temporary array of work item sizes
+/*
+ * Checks whether a given platform supports a given extension.
+ *
+ * Arguments:
+ *  cl_platform_id platform_id  The id of the platform to query
+ *  char* extension             The extension name, as a string.
+ *
+ * Returns 1 if support is detected, else 0.
+ */
+int mcl_platform_supports_extension(cl_platform_id platform_id, const char* extension)
+{
+    // Return from OpenCL
+    cl_int flag;
+
+    // String buffer
+    char buffer[65536];
+
+    // Extensions
+    #ifdef MYOKIT_DEBUG
+    printf("Querying extensions\n");
+    #endif
+    flag = clGetPlatformInfo(platform_id, CL_PLATFORM_EXTENSIONS, sizeof(buffer), buffer, NULL);
+    if(mcl_flag(flag)) return 0;
+    return (strstr(buffer, extension) != NULL);
+}
 
 /*
- * Tidies up if an error occurs in mcl_device_info
+ * Creates and returns a platform information dict, not including a devices
+ * entry.
+ *
+ * Arguments:
+ *  cl_platform_id platform_id  The id of the platform to query
+ *  char* buffer                An initialised string buffer, that can be used
+ *                              to read and write string information.
+ *
+ * Returns NULL and sets an error message if any exception occurs.
  */
-PyObject*
-mcl_device_info_clean()
+PyObject* mcl_info_platform_dict(cl_platform_id platform_id, size_t bufsize, char* buffer)
 {
-    Py_XDECREF(platforms); platforms = NULL;
-    Py_XDECREF(platform); platform = NULL;
-    Py_XDECREF(devices); devices = NULL;
-    Py_XDECREF(device); device = NULL;
-    Py_XDECREF(val); val = NULL;
+    // Return from OpenCL
+    cl_int flag;
+
+    // Platform dict (decref on exception)
+    PyObject* platform;
+
+    // Value to insert into the dict (decref immediatly after)
+    PyObject* val;
+
+    #ifdef MYOKIT_DEBUG
+    printf("Creating platform info dict\n");
+    #endif
+
+    // Create platform dict
+    platform = PyDict_New();
+    if(platform == NULL) {
+        PyErr_SetString(PyExc_Exception, "Unable to create dict for platform info.");
+        return NULL;
+    }
+
+    // Add profile
+    #ifdef MYOKIT_DEBUG
+    printf("  Querying profile\n");
+    #endif
+    flag = clGetPlatformInfo(platform_id, CL_PLATFORM_PROFILE, bufsize, buffer, NULL);
+    if(mcl_flag(flag)) { Py_DECREF(platform); return NULL; }
+    val = PyUnicode_FromString(buffer);
+    PyDict_SetItemString(platform, "profile", val);
+    Py_CLEAR(val);
+
+    // Version
+    #ifdef MYOKIT_DEBUG
+    printf("  Querying version\n");
+    #endif
+    flag = clGetPlatformInfo(platform_id, CL_PLATFORM_VERSION, bufsize, buffer, NULL);
+    if(mcl_flag(flag)) { Py_DECREF(platform); return NULL; }
+    val = PyUnicode_FromString(buffer);
+    PyDict_SetItemString(platform, "version", val);
+    Py_CLEAR(val);
+
+    // Name
+    #ifdef MYOKIT_DEBUG
+    printf("  Querying name\n");
+    #endif
+    flag = clGetPlatformInfo(platform_id, CL_PLATFORM_NAME, bufsize, buffer, NULL);
+    if(mcl_flag(flag)) { Py_DECREF(platform); return NULL; }
+    val = PyUnicode_FromString(buffer);
+    PyDict_SetItemString(platform, "name", val);
+    Py_CLEAR(val);
+
+    // Vendor
+    #ifdef MYOKIT_DEBUG
+    printf("  Querying vendor\n");
+    #endif
+    flag = clGetPlatformInfo(platform_id, CL_PLATFORM_VENDOR, bufsize, buffer, NULL);
+    if(mcl_flag(flag)) { Py_DECREF(platform); return NULL; }
+    val = PyUnicode_FromString(buffer);
+    PyDict_SetItemString(platform, "vendor", val);
+    Py_CLEAR(val);
+
+    // Extensions
+    #ifdef MYOKIT_DEBUG
+    printf("  Querying extensions\n");
+    #endif
+    flag = clGetPlatformInfo(platform_id, CL_PLATFORM_EXTENSIONS, bufsize, buffer, NULL);
+    if(mcl_flag(flag)) { Py_DECREF(platform); return NULL; }
+    val = PyUnicode_FromString(buffer);
+    PyDict_SetItemString(platform, "extensions", val);
+    Py_CLEAR(val);
+
+    // Finished!
+    return platform;
+}
+
+
+/*
+ * Creates and returns a device information dict, not including a devices
+ * entry.
+ *
+ * Arguments:
+ *  cl_device_id device_id  The id of the device to query.
+ *  size_t bufsize          Size of the string `buffer`.
+ *  char* buffer            An initialised string buffer, that can be used to
+ *                          read and write string information.
+ *
+ * Returns NULL and sets an error message if any exception occurs.
+ */
+PyObject* mcl_info_device_dict(cl_device_id device_id, size_t bufsize, char* buffer)
+{
+    // Return from OpenCL
+    cl_int flag;
+
+    // Device dict (decref on exception)
+    PyObject* device;
+
+    // Value to insert into the dict (decref immediately after)
+    PyObject* val;
+
+    // Placeholders for return values
+    cl_uint buf_uint;
+    cl_ulong buf_ulong;
+    size_t buf_size_t;
+
+    // Placeholder for work item sizes, must be freed before returning
+    size_t* work_item_sizes;
+
+    // Item sizes tuple to insert into the dict (decref immediately after)
+    PyObject* items_sizes_tuple;
+
+    // Iteration
+    size_t i;
+
+    #ifdef MYOKIT_DEBUG
+    printf("Creating device info dict\n");
+    #endif
+
+    // Create device dict
+    device = PyDict_New();
+    if(device == NULL) {
+        PyErr_SetString(PyExc_Exception, "Unable to create dict for device info.");
+        return NULL;
+    }
+
+    // Name
+    flag = clGetDeviceInfo(device_id, CL_DEVICE_NAME, bufsize, buffer, NULL);
+    if(mcl_flag(flag)) { Py_DECREF(device); return NULL; }
+    val = PyUnicode_FromString(buffer);
+    PyDict_SetItemString(device, "name", val);
+    Py_CLEAR(val);
+
+    // Vendor
+    flag = clGetDeviceInfo(device_id, CL_DEVICE_VENDOR, bufsize, buffer, NULL);
+    if(mcl_flag(flag)) { Py_DECREF(device); return NULL; }
+    val = PyUnicode_FromString(buffer);
+    PyDict_SetItemString(device, "vendor", val);
+    Py_CLEAR(val);
+
+    // Device version
+    flag = clGetDeviceInfo(device_id, CL_DEVICE_VERSION, bufsize, buffer, NULL);
+    if(mcl_flag(flag)) { Py_DECREF(device); return NULL; }
+    val = PyUnicode_FromString(buffer);
+    PyDict_SetItemString(device, "version", val);
+    Py_CLEAR(val);
+
+    // Driver version
+    flag = clGetDeviceInfo(device_id, CL_DRIVER_VERSION, bufsize, buffer, NULL);
+    if(mcl_flag(flag)) { Py_DECREF(device); return NULL; }
+    val = PyUnicode_FromString(buffer);
+    PyDict_SetItemString(device, "driver", val);
+    Py_CLEAR(val);
+
+    // Clock speed (MHz)
+    flag = clGetDeviceInfo(device_id, CL_DEVICE_MAX_CLOCK_FREQUENCY, sizeof(buf_uint), &buf_uint, NULL);
+    if(mcl_flag(flag)) { Py_DECREF(device); return NULL; }
+    val = PyLong_FromLong(buf_uint);
+    PyDict_SetItemString(device, "clock", val);
+    Py_CLEAR(val);
+
+    // Global memory (bytes)
+    flag = clGetDeviceInfo(device_id, CL_DEVICE_GLOBAL_MEM_SIZE, sizeof(buf_ulong), &buf_ulong, NULL);
+    if(mcl_flag(flag)) { Py_DECREF(device); return NULL; }
+    val = PyLong_FromLong(buf_ulong);
+    PyDict_SetItemString(device, "global", val);
+    Py_CLEAR(val);
+
+    // Local memory (bytes)
+    flag = clGetDeviceInfo(device_id, CL_DEVICE_LOCAL_MEM_SIZE, sizeof(buf_ulong), &buf_ulong, NULL);
+    if(mcl_flag(flag)) { Py_DECREF(device); return NULL; }
+    val = PyLong_FromLong(buf_ulong);
+    PyDict_SetItemString(device, "local", val);
+    Py_CLEAR(val);
+
+    // Const memory (bytes)
+    flag = clGetDeviceInfo(device_id, CL_DEVICE_MAX_CONSTANT_BUFFER_SIZE, sizeof(buf_ulong), &buf_ulong, NULL);
+    if(mcl_flag(flag)) { Py_DECREF(device); return NULL; }
+    val = PyLong_FromLong(buf_ulong);
+    PyDict_SetItemString(device, "const", val);
+    Py_CLEAR(val);
+
+    // Computing units
+    flag = clGetDeviceInfo(device_id, CL_DEVICE_MAX_COMPUTE_UNITS, sizeof(buf_uint), &buf_uint, NULL);
+    if(mcl_flag(flag)) { Py_DECREF(device); return NULL; }
+    val = PyLong_FromLong(buf_uint);
+    PyDict_SetItemString(device, "units", val);
+    Py_CLEAR(val);
+
+    // Max workgroup size
+    flag = clGetDeviceInfo(device_id, CL_DEVICE_MAX_WORK_GROUP_SIZE, sizeof(buf_size_t), &buf_size_t, NULL);
+    if(mcl_flag(flag)) { Py_DECREF(device); return NULL; }
+    val = PyLong_FromLong(buf_size_t);
+    PyDict_SetItemString(device, "groups", val);
+    Py_CLEAR(val);
+
+    // Max workitem sizes
+    flag = clGetDeviceInfo(device_id, CL_DEVICE_MAX_WORK_ITEM_DIMENSIONS, sizeof(buf_uint), &buf_uint, NULL);
+    if(mcl_flag(flag)) { Py_DECREF(device); return NULL; }
+    val = PyLong_FromLong(buf_uint);
+    PyDict_SetItemString(device, "dimensions", val);
+    Py_CLEAR(val);
+
+    work_item_sizes = (size_t*)malloc(buf_uint * sizeof(size_t));
+    flag = clGetDeviceInfo(device_id, CL_DEVICE_MAX_WORK_ITEM_SIZES, buf_uint * sizeof(size_t), work_item_sizes, NULL);
+    if(mcl_flag(flag)) { free(work_item_sizes); Py_DECREF(device); return NULL; }
+    items_sizes_tuple = PyTuple_New((size_t)buf_uint);
+    for (i=0; i<buf_uint; i++) {
+        PyTuple_SetItem(items_sizes_tuple, i, PyLong_FromLong(work_item_sizes[i]));
+    }
     free(work_item_sizes); work_item_sizes = NULL;
-    return 0;
+    PyDict_SetItemString(device, "items", items_sizes_tuple);
+    Py_CLEAR(items_sizes_tuple);
+
+    // Maximum size of a kernel parameter
+    flag = clGetDeviceInfo(device_id, CL_DEVICE_MAX_PARAMETER_SIZE, sizeof(buf_size_t), &buf_size_t, NULL);
+    if(mcl_flag(flag)) { Py_DECREF(device); return NULL; }
+    val = PyLong_FromLong(buf_size_t);
+    PyDict_SetItemString(device, "param", val);
+    Py_CLEAR(val);
+
+    // Done!
+    return device;
 }
 
 /*
  * Returns information about the available OpenCL platforms and devices.
  *
- * Returns a reference to a tuple of platform dicts
+ * Returns a reference to a tuple of platform dicts.
  *
  * platforms = (
  *      dict(platform) {
@@ -542,211 +791,83 @@ mcl_device_info_clean()
  *     )
  */
 PyObject*
-mcl_device_info()
+mcl_info()
 {
-    // Array of platform ids
-    cl_platform_id platform_ids[MCL_MAX_PLATFORMS];
-
-    // Number of platforms
-    cl_uint n_platforms;
-
     // Return from OpenCL
     cl_int flag;
 
-    // Devices & return values from queries
+    // Array of platform ids, number of platforms
+    cl_platform_id platform_ids[MCL_MAX_PLATFORMS];
+    cl_uint n_platforms;
+
+    // Array of device ids, number of ids
     cl_device_id device_ids[MCL_MAX_DEVICES];
     cl_uint n_devices;
-    cl_uint buf_uint;
-    cl_ulong buf_ulong;
-    size_t wgroup_size;
-    size_t max_param;
 
-    // String buffer
+    // String buffer for returned strings
     char buffer[65536];
 
     // Iteration
-    cl_uint i, j, k;
+    cl_uint i, j;
 
-    // Set all pointers used by clean() to null
-    platforms = NULL;
-    platform = NULL;
-    devices = NULL;
-    device = NULL;
-    items = NULL;
-    val = NULL;
-    work_item_sizes = NULL;
+    // Python objects
+    PyObject* platforms;     // Tuple of platform dicts, to be returned
+    PyObject* platform;      // Temporary platform dict
+    PyObject* devices;       // Temporary tuple of devices
+    PyObject* device;        // Temporary device dict
 
     // Get platforms
     n_platforms = 0;
     flag = clGetPlatformIDs(MCL_MAX_PLATFORMS, platform_ids, &n_platforms);
-    if(mcl_flag(flag)) return mcl_device_info_clean();
+    if(mcl_flag(flag)) return NULL;
+    #ifdef MYOKIT_DEBUG
+    printf("Found %d platforms\n", n_platforms);
+    #endif
 
-    // Create platforms tuple
+    // Create platforms tuple, must decref on exception
     platforms = PyTuple_New((size_t)n_platforms);
 
+    // No platforms found
     if (n_platforms == 0) {
-        // No platforms found
         return platforms;
     }
 
     // Check all platforms
     for (i=0; i<n_platforms; i++) {
+
         // Create platform dict
-        platform = PyDict_New();
+        platform = mcl_info_platform_dict(platform_ids[i], sizeof(buffer), buffer);
+        if (platform == NULL) { Py_DECREF(platforms); return NULL; }
 
-        // Profile
-        flag = clGetPlatformInfo(platform_ids[i], CL_PLATFORM_PROFILE, sizeof(buffer), buffer, NULL);
-        if(mcl_flag(flag)) return mcl_device_info_clean();
-        val = PyUnicode_FromString(buffer);
-        PyDict_SetItemString(platform, "profile", val);
-        Py_DECREF(val); val = NULL;
-
-        // Version
-        flag = clGetPlatformInfo(platform_ids[i], CL_PLATFORM_VERSION, sizeof(buffer), buffer, NULL);
-        if(mcl_flag(flag)) return mcl_device_info_clean();
-        val = PyUnicode_FromString(buffer);
-        PyDict_SetItemString(platform, "version", val);
-        Py_DECREF(val); val = NULL;
-
-        // Name
-        flag = clGetPlatformInfo(platform_ids[i], CL_PLATFORM_NAME, sizeof(buffer), buffer, NULL);
-        if(mcl_flag(flag)) return mcl_device_info_clean();
-        val = PyUnicode_FromString(buffer);
-        PyDict_SetItemString(platform, "name", val);
-        Py_DECREF(val); val = NULL;
-
-        // Vendor
-        flag = clGetPlatformInfo(platform_ids[i], CL_PLATFORM_VENDOR, sizeof(buffer), buffer, NULL);
-        if(mcl_flag(flag)) return mcl_device_info_clean();
-        val = PyUnicode_FromString(buffer);
-        PyDict_SetItemString(platform, "vendor", val);
-        Py_DECREF(val); val = NULL;
-
-        // Extensions
-        flag = clGetPlatformInfo(platform_ids[i], CL_PLATFORM_EXTENSIONS, sizeof(buffer), buffer, NULL);
-        if(mcl_flag(flag)) return mcl_device_info_clean();
-        val = PyUnicode_FromString(buffer);
-        PyDict_SetItemString(platform, "extensions", val);
-        Py_DECREF(val); val = NULL;
-
-        // Devices
+        // Count devices
         flag = clGetDeviceIDs(platform_ids[i], CL_DEVICE_TYPE_ALL, MCL_MAX_DEVICES, device_ids, &n_devices);
         if (flag == CL_DEVICE_NOT_FOUND) {
             n_devices = 0;
         } else if(mcl_flag(flag)) {
-            return mcl_device_info_clean();
+            Py_DECREF(platforms);
+            return NULL;
         }
+
+        // Create devices tuple, must decref on exception
         devices = PyTuple_New((size_t)n_devices);
 
+        // Add devices
         for (j=0; j<n_devices; j++) {
-            // Create device dict
-            device = PyDict_New();
 
-            // Name
-            flag = clGetDeviceInfo(device_ids[j], CL_DEVICE_NAME, sizeof(buffer), buffer, NULL);
-            if(mcl_flag(flag)) return mcl_device_info_clean();
-            val = PyUnicode_FromString(buffer);
-            PyDict_SetItemString(device, "name", val);
-            Py_DECREF(val); val = NULL;
+            // Create device dict, must decref on exception
+            device = mcl_info_device_dict(device_ids[j], sizeof(buffer), buffer);
+            if (device == NULL) { Py_DECREF(platforms); Py_DECREF(devices); return NULL; }
 
-            // Vendor
-            flag = clGetDeviceInfo(device_ids[j], CL_DEVICE_VENDOR, sizeof(buffer), buffer, NULL);
-            if(mcl_flag(flag)) return mcl_device_info_clean();
-            val = PyUnicode_FromString(buffer);
-            PyDict_SetItemString(device, "vendor", val);
-            Py_DECREF(val); val = NULL;
-
-            // Device version
-            flag = clGetDeviceInfo(device_ids[j], CL_DEVICE_VERSION, sizeof(buffer), buffer, NULL);
-            if(mcl_flag(flag)) return mcl_device_info_clean();
-            val = PyUnicode_FromString(buffer);
-            PyDict_SetItemString(device, "version", val);
-            Py_DECREF(val); val = NULL;
-
-            // Driver version
-            flag = clGetDeviceInfo(device_ids[j], CL_DRIVER_VERSION, sizeof(buffer), buffer, NULL);
-            if(mcl_flag(flag)) return mcl_device_info_clean();
-            val = PyUnicode_FromString(buffer);
-            PyDict_SetItemString(device, "driver", val);
-            Py_DECREF(val); val = NULL;
-
-            // Clock speed (MHz)
-            flag = clGetDeviceInfo(device_ids[j], CL_DEVICE_MAX_CLOCK_FREQUENCY, sizeof(buf_uint), &buf_uint, NULL);
-            if(mcl_flag(flag)) return mcl_device_info_clean();
-            val = PyLong_FromLong(buf_uint);
-            PyDict_SetItemString(device, "clock", val);
-            Py_DECREF(val); val = NULL;
-
-            // Global memory (bytes)
-            flag = clGetDeviceInfo(device_ids[j], CL_DEVICE_GLOBAL_MEM_SIZE, sizeof(buf_ulong), &buf_ulong, NULL);
-            if(mcl_flag(flag)) return mcl_device_info_clean();
-            val = PyLong_FromLong(buf_ulong);
-            PyDict_SetItemString(device, "global", val);
-            Py_DECREF(val); val = NULL;
-
-            // Local memory (bytes)
-            flag = clGetDeviceInfo(device_ids[j], CL_DEVICE_LOCAL_MEM_SIZE, sizeof(buf_ulong), &buf_ulong, NULL);
-            if(mcl_flag(flag)) return mcl_device_info_clean();
-            val = PyLong_FromLong(buf_ulong);
-            PyDict_SetItemString(device, "local", val);
-            Py_DECREF(val); val = NULL;
-
-            // Const memory (bytes)
-            flag = clGetDeviceInfo(device_ids[j], CL_DEVICE_MAX_CONSTANT_BUFFER_SIZE, sizeof(buf_ulong), &buf_ulong, NULL);
-            if(mcl_flag(flag)) return mcl_device_info_clean();
-            val = PyLong_FromLong(buf_ulong);
-            PyDict_SetItemString(device, "const", val);
-            Py_DECREF(val); val = NULL;
-
-            // Computing units
-            flag = clGetDeviceInfo(device_ids[j], CL_DEVICE_MAX_COMPUTE_UNITS, sizeof(buf_uint), &buf_uint, NULL);
-            if(mcl_flag(flag)) return mcl_device_info_clean();
-            val = PyLong_FromLong(buf_uint);
-            PyDict_SetItemString(device, "units", val);
-            Py_DECREF(val); val = NULL;
-
-            // Max workgroup size
-            flag = clGetDeviceInfo(device_ids[j], CL_DEVICE_MAX_WORK_GROUP_SIZE, sizeof(wgroup_size), &wgroup_size, NULL);
-            if(mcl_flag(flag)) return mcl_device_info_clean();
-            val = PyLong_FromLong(wgroup_size);
-            PyDict_SetItemString(device, "groups", val);
-            Py_DECREF(val); val = NULL;
-
-            // Max workitem sizes
-            flag = clGetDeviceInfo(device_ids[j], CL_DEVICE_MAX_WORK_ITEM_DIMENSIONS, sizeof(buf_uint), &buf_uint, NULL);
-            if(mcl_flag(flag)) return mcl_device_info_clean();
-            val = PyLong_FromLong(buf_uint);
-            PyDict_SetItemString(device, "dimensions", val);
-            Py_DECREF(val); val = NULL;
-
-            work_item_sizes = (size_t*)malloc(buf_uint * sizeof(size_t));
-            flag = clGetDeviceInfo(device_ids[j], CL_DEVICE_MAX_WORK_ITEM_SIZES, buf_uint*sizeof(size_t), work_item_sizes, NULL);
-            if(mcl_flag(flag)) return mcl_device_info_clean();
-            items = PyTuple_New((size_t)buf_uint);
-            for (k=0; k<buf_uint; k++) {
-                PyTuple_SetItem(items, k, PyLong_FromLong(work_item_sizes[k]));
-            }
-            free(work_item_sizes); work_item_sizes = NULL;
-            PyDict_SetItemString(device, "items", items);
-            Py_DECREF(items); items = NULL;
-
-            // Maximum size of a kernel parameter
-            flag = clGetDeviceInfo(device_ids[j], CL_DEVICE_MAX_PARAMETER_SIZE, sizeof(max_param), &max_param, NULL);
-            if(mcl_flag(flag)) return mcl_device_info_clean();
-            val = PyLong_FromLong(max_param);
-            PyDict_SetItemString(device, "param", val);
-            Py_DECREF(val); val = NULL;
-
-            // Add device to devices tuple
+            // Add device to devices tuple (steals reference)
             PyTuple_SetItem(devices, j, device);
             device = NULL;
         }
 
         // Add devices entry to platform dict
         PyDict_SetItemString(platform, "devices", devices);
-        Py_DECREF(devices); devices = NULL;
+        Py_CLEAR(devices);
 
-        // Add platform to platforms tuple
+        // Add platform to platforms tuple (steals reference)
         PyTuple_SetItem(platforms, i, platform);
         platform = NULL;
     }
@@ -755,5 +876,56 @@ mcl_device_info()
     return platforms;
 }
 
+/*
+ * Returns information about the currently selected or default OpenCL platform
+ * and device.
+ *
+ * Arguments:
+ *  PyObject* platform_name  A string naming the selected platform, or None
+ *  PyObject* device_name    A string naming the selected device, or None
+ *
+ * Returns: a platform dict similar to those returned in a tuple by mcl_info,
+ * but with a single "device" entry instead of a "devices" list.
+ */
+PyObject*
+mcl_info_current(
+    PyObject* platform_name,
+    PyObject* device_name)
+{
+    // The current platform and device id
+    cl_platform_id platform_id;
+    cl_device_id device_id;
+
+    // The platform and device dicts (may need decref on exception)
+    PyObject* platform;
+    PyObject* device;
+
+    // String buffer
+    char buffer[65536];
+
+    // Get platform and device id
+    if (mcl_select_device(platform_name, device_name, &platform_id, &device_id)) {
+        return NULL;
+    }
+
+    #ifdef MYOKIT_DEBUG
+    printf("Found platform %d\n", platform_id);
+    printf("Found device %d\n", device_id);
+    #endif
+
+    // Create platform dict
+    platform = mcl_info_platform_dict(platform_id, sizeof(buffer), buffer);
+    if (platform == NULL) return NULL;
+
+    // Add device dict
+    device = mcl_info_device_dict(device_id, sizeof(buffer), buffer);
+    if (device == NULL) { Py_DECREF(platform); return NULL; }
+    PyDict_SetItemString(platform, "device", device);
+    Py_DECREF(device);
+
+    // Return platform dict
+    return platform;
+}
+
 #undef MyokitOpenCL
 #endif
