@@ -481,7 +481,8 @@ class LinearModel(object):
             parameters = [
                 x for x in component.variables(const=True) if x.is_literal()]
             # Sort by qname, using natural sort
-            parameters.sort(key=lambda x: myokit._natural_sort_key(x.qname()))
+            parameters.sort(
+                key=lambda x: myokit.tools.natural_sort_key(x.qname()))
 
         # Get current
         if current is None:
@@ -1848,7 +1849,7 @@ def find_markov_models(model):
 
         # Get sorted list of states for output
         states = list(states)
-        states.sort(key=lambda x: myokit._natural_sort_key(x.qname()))
+        states.sort(key=lambda x: myokit.tools.natural_sort_key(x.qname()))
 
         # Check all members of `group` are a linear combination of states
         try:
