@@ -74,10 +74,17 @@ class ModelTest(unittest.TestCase):
             myokit.DuplicateFunctionArgument, 'already in use',
             m.add_function, 'g', ('a', 'a'), 'a + a')
 
-        # Dot operator is not allowed
+        # Dot operator is not allowed, nor are init or partial
         self.assertRaisesRegex(
             myokit.InvalidFunction, r'dot\(\) operator',
             m.add_function, 'fdot', ('a', ), 'dot(a)')
+        a = myokit.Name('a')
+        self.assertRaisesRegex(
+            myokit.InvalidFunction, r'partial\(\) operator',
+            m.add_function, 'fpart', ('a', ), myokit.PartialDerivative(a, a))
+        self.assertRaisesRegex(
+            myokit.InvalidFunction, r'init\(\) operator',
+            m.add_function, 'finit', ('a', ), myokit.InitialValue(a))
 
         # Unused argument
         self.assertRaisesRegex(
@@ -379,8 +386,8 @@ class ModelTest(unittest.TestCase):
         m2.reserve_unique_name_prefix('cc', 'dd')
         self.assertEqual(m1, m2)
 
-    def test_eval_state_derivatives(self):
-        # Test Model.eval_state_derivatives().
+    def test_evaluate_derivatives(self):
+        # Test Model.evaluate_derivatives().
         model = myokit.Model('m')
         component = model.add_component('comp1')
         t = component.add_variable('time')
@@ -396,19 +403,27 @@ class ModelTest(unittest.TestCase):
         c.promote(3)
         c.set_rhs('b + c')
         model.validate()
-        self.assertEqual(model.eval_state_derivatives(), [1, 4, 5])
+        self.assertEqual(model.evaluate_derivatives(), [1, 4, 5])
         self.assertEqual(
-            model.eval_state_derivatives(state=[1, 1, 2]), [1, 2, 3])
+            model.evaluate_derivatives(state=[1, 1, 2]), [1, 2, 3])
         c.set_rhs('b + c + time')
-        self.assertEqual(model.eval_state_derivatives(), [1, 4, 6])
+        self.assertEqual(model.evaluate_derivatives(), [1, 4, 6])
         self.assertEqual(
-            model.eval_state_derivatives(state=[1, 1, 2], inputs={'time': 0}),
+            model.evaluate_derivatives(state=[1, 1, 2], inputs={'time': 0}),
             [1, 2, 3])
 
+        # Deprecated name
+        with WarningCollector() as w:
+            self.assertEqual(
+                model.eval_state_derivatives(
+                    state=[1, 1, 2], inputs={'time': 0}),
+                [1, 2, 3])
+        self.assertIn('deprecated', w.text())
+
         # Errors
         c.set_rhs('(b + c) / 0')
-        self.assertRaises(myokit.NumericalError, model.eval_state_derivatives)
-        nan = model.eval_state_derivatives(ignore_errors=True)[2]
+        self.assertRaises(myokit.NumericalError, model.evaluate_derivatives)
+        nan = model.evaluate_derivatives(ignore_errors=True)[2]
         self.assertNotEqual(nan, nan)   # x != x is a nan test...
 
     def test_format_state(self):
@@ -523,7 +538,7 @@ class ModelTest(unittest.TestCase):
 
         # Test with state and precision argument
         # Ignoring some of the middle digits, as they differ on some (but not
-        # all!) travis builds.
+        # all!) CI builds.
         out = m.format_state_derivatives(
             state1, precision=myokit.SINGLE_PRECISION).splitlines()
         self.assertEqual(len(out), 8)
@@ -648,6 +663,468 @@ class ModelTest(unittest.TestCase):
         self.assertFalse(
             m.has_variables(const=False, state=False, bound=False))
 
+    def test_import_component(self):
+        # Test :meth: 'import_component()'.
+
+        # Source model, to import stuff from
+        ms = myokit.parse_model('''
+            [[model]]
+            p.b = 0.2
+            q.e = 0.2
+
+            [e]
+            t = 0 [s] bind time
+                in [s]
+            g = 0
+                label this_is_g
+            h = 4
+
+            # Independent (except for time)
+            [p]
+            a = 1 [1/s]
+                in [1/s]
+            # Comments should be stripped out during parsing, so this is OK.
+            dot(b) = a * c
+                in [m]
+                c = b * 2
+                    in [m]
+
+            # Requires mapping, has aliases and an unused alias
+            [q]
+            use p.a, p.b
+            use e.h
+            dot(e) = d * b
+                in [m*A]
+            f = 3 * dot(b)
+                in [m/s]
+            d = 0.2 [A] * a
+                in [A/s]
+
+        ''')
+        ms.validate()
+        ms.check_units(myokit.UNIT_STRICT)
+
+        # Make copy of ms, to ensure nothing is altered in original model
+        ms_unaltered = ms.clone()
+
+        # Import independent component into empty model
+        m1 = myokit.Model()
+        m1.import_component(ms['p'])
+        self.assertTrue(m1.has_component('p'))
+        self.assertFalse(m1['p'] is ms['p'])
+        self.assertEqual(m1['p'].code(), ms['p'].code())
+        self.assertEqual(ms, ms_unaltered)
+
+        # Import a second time, without renaming
+        m1.import_component, ms['p']  # Check errors happen before changes
+        m1_unaltered = m1.clone()
+        self.assertRaises(myokit.DuplicateName, m1.import_component, ms['p'])
+        self.assertEqual(m1, m1_unaltered)
+
+        # Import a second time, and rename
+        m1.import_component(ms['p'], new_name='p2')
+        self.assertTrue(m1.has_component('p2'))
+        self.assertFalse(m1['p2'] is ms['p'])
+        self.assertFalse(m1['p2'] is m1['p'])
+        cs = '\n'.join((ms['p'].code().splitlines())[1:])
+        c1 = '\n'.join((m1['p2'].code().splitlines())[1:])
+        self.assertEqual(cs, c1)
+        self.assertEqual(ms, ms_unaltered)
+
+        # Import independent component with labels and bindings
+        m1.import_component(ms['e'])
+        self.assertTrue(m1.has_component('e'))
+        self.assertFalse(m1['e'] is ms['e'])
+        self.assertEqual(m1['e'].code(), ms['e'].code())
+        self.assertEqual(m1.label('this_is_g'), m1.get('e.g'))
+        self.assertEqual(m1.binding('time'), m1.get('e.t'))
+        self.assertEqual(ms, ms_unaltered)
+
+        # Now that it has a time variable, m1 should be valid
+        m1.validate()
+        self.assertTrue(m1.is_valid())
+
+        # Re-importing e is not allowed, as it leads to double bindings and/or
+        # labels
+        m1_unaltered = m1.clone()
+        ms.time().set_binding(None)
+        self.assertRaisesRegex(
+            myokit.InvalidLabelError, 'label "this_is_g"',
+            m1.import_component, ms['e'], new_name='dinosaur')
+        self.assertEqual(m1, m1_unaltered)
+        ms.get('e.t').set_binding('time')
+        ms.label('this_is_g').set_label(None)
+        self.assertRaisesRegex(
+            myokit.InvalidBindingError, 'binding "time"',
+            m1.import_component, ms['e'], new_name='hello')
+        self.assertEqual(m1, m1_unaltered)
+        ms = ms_unaltered.clone()
+
+        # Import r, using a custom variable mapping
+        var_map = {'p.a': 'p.a', 'p.b': 'p2.b', 'e.h': 'e.h'}
+        m1.import_component(ms['q'], var_map=var_map)
+        self.assertTrue(m1.has_component('q'))
+        self.assertFalse(m1['q'] is ms['q'])
+        self.assertEqual(m1['q'].alias('a'), m1.get('p.a'))
+        self.assertEqual(m1['q'].alias('b'), m1.get('p2.b'))
+        self.assertEqual(m1['q'].alias('h'), m1.get('e.h'))
+        cs = '\n'.join((ms['p'].code().splitlines())[4:])
+        c1 = '\n'.join((m1['p2'].code().splitlines())[4:])
+        self.assertEqual(cs, c1)
+        self.assertEqual(ms, ms_unaltered)
+
+        # Import r, using a label for p.b
+        ms.get('p.b').set_label('this_is_b')
+        m1.get('p2.b').set_label('this_is_b')
+        var_map = {'p.a': 'p.a', 'e.h': 'e.h'}
+        m1.import_component(ms['q'], var_map=var_map, new_name='q2')
+        self.assertTrue(m1.has_component('q2'))
+        self.assertFalse(m1['q2'] is ms['q'])
+        self.assertFalse(m1['q2'] is m1['q'])
+        self.assertEqual(m1['q2'].alias('a'), m1.get('p.a'))
+        self.assertEqual(m1['q2'].alias('b'), m1.get('p2.b'))
+        cs = '\n'.join((ms['p'].code().splitlines())[3:])
+        c1 = '\n'.join((m1['p2'].code().splitlines())[3:])
+        self.assertEqual(cs, c1)
+        ms.get('p.b').set_label(None)
+        m1.get('p2.b').set_label(None)
+        self.assertEqual(ms, ms_unaltered)
+
+        # Import r, using a binding for e.h
+        ms.get('e.h').set_binding('this_is_h')
+        m1.get('e.h').set_binding('this_is_h')
+        var_map = {'p.a': 'p.a', 'p.b': 'p2.b'}
+        m1.import_component(ms['q'], var_map=var_map, new_name='q3')
+        self.assertTrue(m1.has_component('q3'))
+        self.assertFalse(m1['q3'] is ms['q'])
+        self.assertFalse(m1['q3'] is m1['q'])
+        self.assertEqual(m1['q3'].alias('a'), m1.get('p.a'))
+        self.assertEqual(m1['q3'].alias('b'), m1.get('p2.b'))
+        cs = '\n'.join((ms['p'].code().splitlines())[3:])
+        c1 = '\n'.join((m1['p2'].code().splitlines())[3:])
+        self.assertEqual(cs, c1)
+        ms.get('e.h').set_binding(None)
+        m1.get('e.h').set_binding(None)
+        self.assertEqual(ms, ms_unaltered)
+
+        # Import r, using (partial) name mapping
+        var_map = {'p.a': 'p2.a'}
+        m1.import_component(
+            ms['q'], var_map=var_map, new_name='q4', allow_name_mapping=True)
+        self.assertTrue(m1.has_component('q3'))
+        self.assertFalse(m1['q3'] is ms['q'])
+        self.assertFalse(m1['q4'] is m1['q'])
+        self.assertEqual(m1['q4'].alias('a'), m1.get('p2.a'))
+        cs = '\n'.join((ms['p'].code().splitlines())[2:])
+        c1 = '\n'.join((m1['p2'].code().splitlines())[2:])
+        self.assertEqual(cs, c1)
+        self.assertEqual(ms, ms_unaltered)
+
+        # Try and fail to import r without a mapping
+        m1_unaltered = m1.clone()
+        self.assertRaises(
+            myokit.VariableMappingError,
+            m1.import_component, ms['q'], new_name='q9')
+        self.assertEqual(m1, m1_unaltered)
+        self.assertEqual(ms, ms_unaltered)
+
+        # With an invalid mapping
+        self.assertRaisesRegex(
+            TypeError, 'dict or None',
+            m1.import_component, ms['q'], new_name='q9', var_map=[3])
+        self.assertEqual(m1, m1_unaltered)
+        self.assertRaisesRegex(
+            TypeError, 'objects or fully qualified',
+            m1.import_component, ms['q'], new_name='q9',
+            var_map={'p.a': 123, 'p.b': 'p.b', 'e.h': 'e.h'})
+        self.assertEqual(m1, m1_unaltered)
+        self.assertRaisesRegex(
+            TypeError, 'objects or fully qualified',
+            m1.import_component, ms['q'], new_name='q9',
+            var_map={345: 'p.a', 'p.b': 'p.b', 'e.h': 'e.h'})
+        self.assertEqual(m1, m1_unaltered)
+        self.assertRaisesRegex(
+            myokit.VariableMappingError, 'Multiple variables map',
+            m1.import_component, ms['q'], new_name='q9',
+            var_map={'p.a': 'p.a', 'p.b': 'p.a', 'e.h': 'e.h'})
+        self.assertEqual(m1, m1_unaltered)
+        self.assertEqual(ms, ms_unaltered)
+
+        # With variables from another model or that don't exist
+        self.assertRaisesRegex(
+            myokit.VariableMappingError, 'was not found in this model',
+            m1.import_component, ms['q'], new_name='q9',
+            var_map={'p.a': 'one.two', 'p.b': 'p.b', 'e.h': 'e.h'})
+        self.assertEqual(m1, m1_unaltered)
+        self.assertEqual(ms, ms_unaltered)
+
+        self.assertRaisesRegex(
+            myokit.VariableMappingError, 'was not found in the source model',
+            m1.import_component, ms['q'], new_name='q9',
+            var_map={'ppp.aaa': 'p.a', 'p.b': 'p.b', 'e.h': 'e.h'})
+        self.assertEqual(m1, m1_unaltered)
+        self.assertEqual(ms, ms_unaltered)
+
+        v = myokit.Model('a').add_component('p').add_variable('b')
+        self.assertRaisesRegex(
+            myokit.VariableMappingError, 'is not part of this model',
+            m1.import_component, ms['q'], new_name='q9',
+            var_map={'p.a': v, 'p.b': 'p.b'})
+        self.assertEqual(m1, m1_unaltered)
+        self.assertEqual(ms, ms_unaltered)
+
+        self.assertRaisesRegex(
+            myokit.VariableMappingError, 'is not part of the source model',
+            m1.import_component, ms['q'], new_name='q9',
+            var_map={v: 'p.a', 'p.b': 'p.b', 'e.h': 'e.h'})
+        self.assertEqual(m1, m1_unaltered)
+        self.assertEqual(ms, ms_unaltered)
+
+        # Incomplete mapping
+        self.assertRaisesRegex(
+            myokit.VariableMappingError, 'cannot be mapped',
+            m1.import_component, ms['q'], new_name='q9', var_map={})
+        self.assertEqual(m1, m1_unaltered)
+        self.assertEqual(ms, ms_unaltered)
+
+        # Import something that's not a component
+        self.assertRaisesRegex(
+            TypeError, 'myokit.Component',
+            m1.import_component, 'q')
+
+        # Import your own components
+        self.assertRaisesRegex(
+            ValueError, 'part of this model',
+            m1.import_component, m1['p'], new_name='abc')
+
+    def test_import_component_units(self):
+        # Test :meth: 'import_component()' with unit conversion.
+
+        # Source model, to import stuff from
+        ms = myokit.parse_model('''
+            [[model]]
+            p.c = 0.1
+            r.f = 0.2
+
+            [e]
+            t = 0 [s] bind time
+                in [s]
+
+            # Independent (except for time)
+            [p]
+            a = 1 [m] in [m]
+            b = 10 [s] in [s]
+            dot(c) = 0.2 [m/s]
+                in [m]
+
+            # Requires mapping, doesn't have states but has dot() reference
+            [q]
+            d = 3.2 [m/s] + dot(p.c)
+                in [m/s]
+            e = p.a + p.c + 0.1 [m]
+                in [m]
+
+            # Requires mapping, has a new state
+            [r]
+            use p.a, p.b
+            dot(f) = a / b
+                in [m]
+
+            # No time units used
+            [s]
+            x = 3 [kg] in [kg]
+        ''')
+        ms.validate()
+        ms.check_units(myokit.UNIT_STRICT)
+
+        # Make copy of ms, to ensure nothing is altered in original model
+        ms_unaltered = ms.clone()
+
+        # Conversion helpers
+        vm = {'p.a': 'p.a', 'p.b': 'p.b', 'p.c': 'p.c'}
+        s2ms = myokit.Number(1000, myokit.units.ms / myokit.units.s)
+        mm2m = myokit.Number(0.001, myokit.units.m / myokit.units.mm)
+
+        # Target model to import stuff into, with different time units
+        m1 = myokit.parse_model('''
+            [[model]]
+
+            [e]
+            t = 0 [ms] bind time
+                in [ms]
+        ''')
+        m1.validate()
+        m1.check_units(myokit.UNIT_STRICT)
+
+        # Import p, should be same
+        m1.import_component(ms['p'], convert_units=True)
+
+        # Import q, should be same except for dot() expression
+        m1.import_component(ms['q'], var_map=vm, convert_units=True)
+        self.assertEqual(ms, ms_unaltered)
+        self.assertIn('q', m1)
+        self.assertEqual(len(m1['q']), 2)
+        self.assertEqual(m1.get('q.e').code(), ms.get('q.e').code())
+        self.assertEqual(m1.get('q.d').unit(), ms.get('q.d').unit())
+        self.assertEqual(
+            m1.get('q.d').rhs().code(),
+            myokit.Plus(myokit.Number(3.2, 'm/s'), myokit.Multiply(
+                myokit.Derivative(myokit.Name(m1.get('p.c'))), s2ms)
+            ).code())
+
+        # Import r, should be same except for the state's RHS
+        m1.import_component(ms['r'], var_map=vm, convert_units=True)
+        self.assertEqual(ms, ms_unaltered)
+        self.assertIn('r', m1)
+        self.assertEqual(len(m1['r']), 1)
+        self.assertIn('f', m1['r'])
+        self.assertTrue(m1.get('r.f').is_state())
+        self.assertEqual(m1.get('r.f').unit(), ms.get('r.f').unit())
+        self.assertEqual(
+            m1.get('r.f').state_value(), ms.get('r.f').state_value())
+        self.assertEqual(
+            m1.get('r.f').rhs().code(),
+            myokit.Multiply(ms.get('r.f').rhs(), s2ms).code())
+
+        # Target model with different space units
+        m1 = myokit.parse_model('''
+            [[model]]
+            p.c = 0.1
+
+            [e]
+            time = 0 [s] bind time
+                in [s]
+
+            [p]
+            a = 1 [mm] in [mm]
+            b = 10 [s] in [s]
+            dot(c) = 0.2 [mm/s]
+                in [mm]
+        ''')
+        m1.validate()
+        m1.check_units(myokit.UNIT_STRICT)
+
+        # Import q, converting p.a and dot(p.c)
+        m1.import_component(ms['q'], var_map=vm, convert_units=True)
+        self.assertEqual(ms, ms_unaltered)
+        self.assertIn('q', m1)
+        self.assertEqual(len(m1['q']), 2)
+        self.assertEqual(m1.get('q.d').unit(), ms.get('q.d').unit())
+        self.assertEqual(
+            m1.get('q.d').rhs().code(),
+            myokit.Plus(
+                myokit.Number(3.2, 'm/s'),
+                myokit.Multiply(
+                    myokit.Derivative(myokit.Name(m1.get('p.c'))), mm2m)
+            ).code())
+        self.assertEqual(m1.get('q.e').unit(), ms.get('q.e').unit())
+        self.assertEqual(
+            m1.get('q.e').rhs().code(),
+            myokit.Plus(myokit.Plus(
+                myokit.Multiply(myokit.Name(m1.get('p.a')), mm2m),
+                myokit.Multiply(myokit.Name(m1.get('p.c')), mm2m)),
+                myokit.Number(0.1, 'm')).code())
+
+        # Target model with different time and space units
+        m1 = myokit.parse_model('''
+            [[model]]
+            p.c = 0.1
+
+            [e]
+            time = 0 [ms] bind time
+                in [ms]
+
+            [p]
+            a = 1 [mm] in [mm]
+            b = 10 [ms] in [ms]
+            dot(c) = 0.2 [mm/ms]
+                in [mm]
+        ''')
+        m1.validate()
+        m1.check_units(myokit.UNIT_STRICT)
+
+        # Import q, converting p.a and dot(p.c)
+        m1.import_component(ms['q'], var_map=vm, convert_units=True)
+        self.assertEqual(ms, ms_unaltered)
+        self.assertIn('q', m1)
+        self.assertEqual(len(m1['q']), 2)
+        self.assertEqual(m1.get('q.d').unit(), ms.get('q.d').unit())
+        self.assertEqual(
+            m1.get('q.d').rhs().code(),
+            myokit.Plus(
+                myokit.Number(3.2, 'm/s'),
+                myokit.Derivative(myokit.Name(m1.get('p.c')))
+            ).code())
+        self.assertEqual(m1.get('q.e').unit(), ms.get('q.e').unit())
+        self.assertEqual(
+            m1.get('q.e').rhs().code(),
+            myokit.Plus(myokit.Plus(
+                myokit.Multiply(myokit.Name(m1.get('p.a')), mm2m),
+                myokit.Multiply(myokit.Name(m1.get('p.c')), mm2m)),
+                myokit.Number(0.1, 'm')).code())
+
+        # Attempt import with incompatible time units 1:
+        # Component that defines a derivative in m/s
+        m1 = myokit.parse_model('''
+            [[model]]
+
+            [e]
+            time = 0 [A] bind time
+                in [A]
+        ''')
+        m1.validate()
+        m1.check_units(myokit.UNIT_STRICT)
+        self.assertRaisesRegex(
+            myokit.VariableMappingError, 'time variables',
+            m1.import_component, ms['p'], convert_units=True)
+        self.assertEqual(ms, ms_unaltered)
+
+        # Attempt import with incompatible time units 1:
+        # Component that uses a dot expression
+        m1 = myokit.parse_model('''
+            [[model]]
+            p.c = 0
+
+            [e]
+            time = 0 [A] bind time
+                in [A]
+
+            [p]
+            a = 1 [m] in [m]
+            dot(c) = 0.2 [m/A]
+                in [m]
+        ''')
+        m1.validate()
+        m1.check_units(myokit.UNIT_STRICT)
+        vm = {'p.a': 'p.a', 'p.c': 'p.c'}
+        self.assertRaisesRegex(
+            myokit.VariableMappingError, 'time variables',
+            m1.import_component, ms['q'], var_map=vm, convert_units=True)
+        self.assertEqual(ms, ms_unaltered)
+
+        # Attempt import with incompatible mapped (space) units
+        m1 = myokit.parse_model('''
+            [[model]]
+            p.c = 0
+
+            [e]
+            time = 0 [s] bind time
+                in [s]
+
+            [p]
+            a = 1 [V] in [V]
+            b = 10 [s] in [s]
+            dot(c) = 0.2 [V/s]
+                in [V]
+        ''')
+        m1.validate()
+        m1.check_units(myokit.UNIT_STRICT)
+        vm = {'p.a': 'p.a', 'p.b': 'p.b', 'p.c': 'p.c'}
+        self.assertRaisesRegex(
+            myokit.VariableMappingError, 'Unable to convert',
+            m1.import_component, ms['q'], var_map=vm, convert_units=True)
+        self.assertEqual(ms, ms_unaltered)
+
     def test_item_at_text_position(self):
         # Test :meth:`Model.item_at_text_position()`.
 
@@ -1006,8 +1483,8 @@ class ModelTest(unittest.TestCase):
         self.assertEqual(m1.code(), m2.code())
 
         # Assert models both produce the same derivatives
-        dy1 = m1.eval_state_derivatives()
-        dy2 = m2.eval_state_derivatives()
+        dy1 = m1.evaluate_derivatives()
+        dy2 = m2.evaluate_derivatives()
         self.assertEqual(dy1, dy2)
 
         # Test time unit is None
@@ -1118,12 +1595,6 @@ class ModelTest(unittest.TestCase):
         m.resolve_interdependent_components()
         self.assertEqual(m.count_components(), 3)
 
-        # Test deprecated name
-        with WarningCollector() as wc:
-            m.merge_interdependent_components()
-        self.assertIn('deprecated', wc.text())
-        self.assertEqual(m.count_components(), 3)
-
         # Create interdependent components
         v.set_rhs('3 - c3.x')
         w.set_rhs('1 - c2.v')
@@ -1239,11 +1710,6 @@ class ModelTest(unittest.TestCase):
         self.assertEqual(m.show_line_of(v, raw=True), 91)
         self.assertIsNone(m2.show_line_of(v2, raw=True))
 
-        # Test deprecated alias
-        with WarningCollector() as wc:
-            m.show_line(m.get('ina.INa'))
-        self.assertIn('deprecated', wc.text())
-
     def test_str(self):
         # Test conversion to string
 
